const {Command, flags} = require('@oclif/command')
const {cli} = require('cli-ux')
const axios = require('axios')
const chalk = require('chalk')
const inquirer = require('inquirer')
const prompt = inquirer.createPromptModule()
const execa = require('execa')
const listr = require('listr')
const fs = require('fs')

// activate the .env config
require('dotenv').config();


class PagesPayrollCliCommand extends Command {

  
  async run() {
    
    const {flags, args} = this.parse(PagesPayrollCliCommand)
    const headers = {
      headers: {
        'Content-Type': 'application/json',
        'Authorization': `Bearer ${process.env.DO_ACCESS_TOKEN}`
      }
    }

    let hostname = flags.hostname || args.hostname || false
    
    const _self = this
    
    if(!hostname || !hostname.match(/^[a-z]{3,15}$/i)){
      const response = await prompt([
        {
          type: 'input',
          name: 'hostname',
          message: chalk`Enter the {bold.yellow hostname}: `,
          validate: function(value) {
            return value.match(/^[a-z]{3,15}$/i) ? true : 'Please enter a valid hostname!';
          },
          choices: [],
        }
      ])
      hostname = response.hostname
    }
    
    // Steps for Pages Payroll Deployment
    const tasks = new listr(
      [
        // Create a new A record for new PagesPayroll Client
        {
          title: `Create a new subdomain - ${hostname}.${process.env.DOMAIN_NAME}`,
          task: () => {
            return new listr([
              {
                title: 'Checking if subdomain already exists',
                task: async (ctx, task) => {
                  await axios.get(`https://api.digitalocean.com/v2/domains/${process.env.DOMAIN_NAME}/records`, headers)
                    .then(function (response) {
                        // handle success
                        if(response.data.domain_records.some((record) => record.name === hostname)){
                          ctx.hostname_exists = true
                        }
                        else{
                          ctx.hostname_exists = false
                        }
                    })
                    .catch(function (error) {
                        // handle error
                        throw new Error(error.response.data.message)
                    })
                }
              },
              {
                title: `Create new subdomain - ${hostname}.${process.env.DOMAIN_NAME}`,
                skip: (ctx) => {
                  return ctx.hostname_exists ? `Subdomain - ${hostname}.${process.env.DOMAIN_NAME} already exists!` : false
                },
                task: async (ctx, task) => {
                  const data = {
                    type: 'A',
                    name: hostname,
                    data: process.env.DOMAIN_IP
                  }
                  await axios.post(`https://api.digitalocean.com/v2/domains/${process.env.DOMAIN_NAME}/records`, data, headers)
                    .then(function (response) {
                        // handle success
                        return `${hostname}.${process.env.DOMAIN_NAME} created with ID: ${response.data.id}`
                    })
                    .catch(function (error) {
                        // handle error
                        throw new Error(error.response.data.message)
                    })
                }
              }
            ])
          }
        },
        // Create virtual host conf in apache2 sites-available 
        {
          title: `Create virtual host configurations - ${hostname}.${process.env.DOMAIN_NAME}.conf`,
          task: () => {
            return new listr([
              {
                title: `Create a copy of the demo.${process.env.DOMAIN_NAME}.conf`,
                task: async (ctx, task) => {
                  // var dir = `${process.env.APACHE_DIR}`;
                  // if (!fs.existsSync(dir)){
                  //   fs.mkdirSync(dir);
                  // }
                  await execa('cp', [`${process.env.APACHE}/demo.${process.env.DOMAIN_NAME}.conf`, `${process.env.APACHE}/${hostname}.${process.env.DOMAIN_NAME}.conf`])
                }
              },
              {
                title: `Replace 'demo.${process.env.DOMAIN_NAME}' with '${hostname}.${process.env.DOMAIN_NAME}' in the conf file`,
                task: async (ctx, task) => {
                  let filename = `${process.env.APACHE}/${hostname}.${process.env.DOMAIN_NAME}.conf`
                  fs.readFile(filename, 'utf8', function (err, data) {
                    if (err) {
                      return console.log(err);
                    }
                    var result = data.replace(/demo/g, hostname)

                    fs.writeFile(filename, result, 'utf8', function (err) {
                      if (err) return console.log(err);
                    })
                  })
                  return `conf file updated`
                }
              },
              {
                title: `Enable the ${hostname}.${process.env.DOMAIN_NAME}.conf`,
                task: async (ctx, task) => {
                  await execa('a2ensite', [`${process.env.APACHE}/${hostname}.${process.env.DOMAIN_NAME}.conf`])
                }
              },
              {
                title: `Restart apache2`,
                task: async (ctx, task) => {
                  await execa('systemctl', ['restart', 'apache2'])
                }
              },
              {
                title: `Restart apache2 service`,
                task: async (ctx, task) => {
                  await execa('service', ['apache2', 'restart'])
                }
              }
            ])
          }
        },
        // Create a new subdirectory and update the current working directory & Clone Code from BitBucket
        {
          title: `Create a new subdirectory - ${process.env.HOME_DIR}/${hostname}.${process.env.DOMAIN_NAME}`,
          task: () => {
            return new listr([
              {
                title: `Create a new subdirectory - ${process.env.HOME_DIR}/${hostname}.${process.env.DOMAIN_NAME}`,
                task: async (ctx, task) => {
                  var dir = `${process.env.HOME_DIR}/${hostname}.${process.env.DOMAIN_NAME}`;
                  if (!fs.existsSync(dir)){
                    fs.mkdirSync(dir);
                  }
                }
              },
              {
                title: `Replace 'demo.${process.env.DOMAIN_NAME}' with '${hostname}.${process.env.DOMAIN_NAME}' in the conf file`,
                task: async (ctx, task) => {
                  // change directory to the newly created subdirectory
                  process.chdir(`${process.env.HOME_DIR}/${hostname}.${process.env.DOMAIN_NAME}`);
                  return `Changed directory to: ${process.cwd()}`
                }
              },
              {
                title: `Clone the Bitbucket Repository`,
                task: async (ctx, task) => {
                  await execa('git', ['clone', process.env.REPOSITORY_URL, '.'])
                }
              }
            ])
          }
        },
        // Setup Laravel Project
        {
          title: `Clone the BitBucket Repo - ${process.env.HOME_DIR}/${hostname}.${process.env.DOMAIN_NAME}`,
          task: () => {
            return new listr([
              {
                title: `Clone the Git Repository`,
                task: async (ctx, task) => {
                  await execa('git', ['clone', process.env.REPOSITORY_URL, '.'])
                }
              }
            ])
          }
        }
      ]
    )

    tasks.run().catch(err => {
      // this.error(err);
    });
    
    // Steps for Pages Payroll Deployment
    // 1. Create a new A record for new PagesPayroll Client
    // 2. Create a apache2.conf file in /etc/apache2/sites-available/ 
    // 3. enable site : a2ensite <conf>
    // 4. Edit the directory in the above created confirmation file

    // 5. Create a new subfolder in the name of the new Entity
    // 6. Create a new Database for the new Entity
    // 7. Close the ‘pages-payroll’ to the newly created directory
    // 8. Make a copy of the .env.example file to .env
    // 9. Make the necessary changes in the .env file
    // 10. Run Commands:
    //   - composer install

    //   - sudo chown -R $USER:www-data storage
    //   - sudo chown -R $USER:www-data bootstrap/cache
  
    //   - chmod -R 775 storage
    //   - chmod -R 775 bootstrap/cache
  
    //   - php artisan migrate
    //   - php artisan passport:install
    //   - php artisan route:cache
    //   - php artisan config:cache

    // this.log(`hey! from ./src/index.js`)
  }
}

PagesPayrollCliCommand.description = `Describe the command here
...
Extra documentation goes here
`

PagesPayrollCliCommand.flags = {
  // add --version flag to show CLI version
  version: flags.version({char: 'v'}),
  // add --help flag to show CLI version
  help: flags.help({char: 'h'}),
  hostname: flags.string({char: 'hn'})
}

PagesPayrollCliCommand.args = [
  {
    name: 'hostname',             // name of arg to show in help and reference with args[name]
    required: false,              // make the arg required with `required: true`
    description: 'hostname for pagespayroll.app subdomain', // help description
    hidden: false,                // hide this arg from help
  }
]

module.exports = PagesPayrollCliCommand
